@@ -2,8 +2,8 @@
 import os
 
 # Replaced with the current commit when building the wheels.
-__commit__ = 'f6200d6e2bb8bcef91efdb903669a7d5e1acdbf9'
-__version__ = '0.2.1'
+__commit__ = '23c3f67564f955af4670534e8e9d5ea19ad97e6d'
+__version__ = '0.2.2'
 __root_dir__ = os.path.dirname(os.path.abspath(__file__))
 
 # Keep this order to avoid cyclic imports
@@ -16,12 +16,12 @@ from sky.execution import launch, exec, spot_launch  # pylint: disable=redefined
 from sky.resources import Resources
 from sky.task import Task
 from sky.optimizer import Optimizer, OptimizeTarget
-from sky.data import Storage, StoreType
+from sky.data import Storage, StorageMode, StoreType
 from sky.global_user_state import ClusterStatus
 from sky.skylet.job_lib import JobStatus
 from sky.core import (status, start, stop, down, autostop, queue, cancel,
-                      tail_logs, download_logs, job_status, spot_status,
-                      spot_cancel, storage_ls, storage_delete)
+                      tail_logs, download_logs, job_status, spot_queue,
+                      spot_status, spot_cancel, storage_ls, storage_delete)
 
 # Aliases.
 AWS = clouds.AWS
@@ -43,6 +43,7 @@ __all__ = [
     'list_accelerators',
     '__root_dir__',
     'Storage',
+    'StorageMode',
     'StoreType',
     'ClusterStatus',
     'JobStatus',
@@ -67,7 +68,8 @@ __all__ = [
     'download_logs',
     'job_status',
     # core APIs Spot Job Management
-    'spot_status',
+    'spot_queue',
+    'spot_status',  # Deprecated (alias for spot_queue)
     'spot_cancel',
     # core APIs Storage Management
     'storage_ls',
