@@ -6,13 +6,18 @@ import typing
 from typing import Dict, Iterator, List, Optional, Tuple
 
 from sky import clouds
+from sky import exceptions
+from sky import sky_logging
 from sky.adaptors import azure
 from sky.clouds import service_catalog
+from sky.utils import common_utils
 from sky.utils import ux_utils
 
 if typing.TYPE_CHECKING:
     from sky import resources
 
+logger = sky_logging.init_logger(__name__)
+
 # Minimum set of files under ~/.azure that grant Azure access.
 _CREDENTIAL_FILES = [
     'azureProfile.json',
@@ -21,6 +26,8 @@ _CREDENTIAL_FILES = [
     'msal_token_cache.json',
 ]
 
+_MAX_IDENTITY_FETCH_RETRY = 10
+
 
 def _run_output(cmd):
     proc = subprocess.run(cmd,
@@ -38,13 +45,23 @@ class Azure(clouds.Cloud):
     _REPR = 'Azure'
     _regions: List[clouds.Region] = []
 
-    def instance_type_to_hourly_cost(self, instance_type, use_spot):
+    def instance_type_to_hourly_cost(self,
+                                     instance_type: str,
+                                     use_spot: bool,
+                                     region: Optional[str] = None,
+                                     zone: Optional[str] = None) -> float:
         return service_catalog.get_hourly_cost(instance_type,
-                                               region=None,
                                                use_spot=use_spot,
+                                               region=region,
+                                               zone=zone,
                                                clouds='azure')
 
-    def accelerators_to_hourly_cost(self, accelerators, use_spot):
+    def accelerators_to_hourly_cost(self,
+                                    accelerators: Dict[str, int],
+                                    use_spot: bool,
+                                    region: Optional[str] = None,
+                                    zone: Optional[str] = None) -> float:
+        del accelerators, use_spot, region, zone  # unused
         # Azure includes accelerators as part of the instance type.
         # Implementing this is also necessary for e.g., the instance may have 4
         # GPUs, while the task specifies to use 1 GPU.
@@ -77,14 +94,18 @@ class Azure(clouds.Cloud):
         return isinstance(other, Azure)
 
     @classmethod
-    def get_default_instance_type(cls,
-                                  accelerators: Optional[Dict[str, int]] = None
-                                 ) -> str:
-        del accelerators
+    def get_default_instance_type(cls) -> str:
         # 8 vCpus, 32 GB RAM.  Prev-gen (as of 2021) general purpose.
         return 'Standard_D8_v4'
 
     def _get_image_config(self, gen_version, instance_type):
+        # az vm image list \
+        #  --publisher microsoft-dsvm --all --output table
+        # nvidia-driver: 495.29.05, cuda: 11.5
+
+        # The latest image 2022.09.14/2022.08.11/22.06.10/22.05.11/
+        # 22.04.27/22.04.05 has even older nvidia driver 470.57.02,
+        # cuda: 11.4
         image_config = {
             'image_publisher': 'microsoft-dsvm',
             'image_offer': 'ubuntu-2004',
@@ -129,22 +150,40 @@ class Azure(clouds.Cloud):
             ]
         return cls._regions
 
+    @classmethod
+    def regions_with_offering(cls, instance_type: Optional[str],
+                              accelerators: Optional[Dict[str, int]],
+                              use_spot: bool, region: Optional[str],
+                              zone: Optional[str]) -> List[clouds.Region]:
+        del accelerators  # unused
+        if instance_type is None:
+            # Fall back to default regions
+            regions = cls.regions()
+        else:
+            regions = service_catalog.get_region_zones_for_instance_type(
+                instance_type, use_spot, 'azure')
+
+        if region is not None:
+            regions = [r for r in regions if r.name == region]
+        if zone is not None:
+            for r in regions:
+                r.set_zones([z for z in r.zones if z.name == zone])
+            regions = [r for r in regions if r.zones]
+        return regions
+
     @classmethod
     def region_zones_provision_loop(
         cls,
         *,
         instance_type: Optional[str] = None,
         accelerators: Optional[Dict[str, int]] = None,
-        use_spot: bool,
+        use_spot: bool = False,
     ) -> Iterator[Tuple[clouds.Region, List[clouds.Zone]]]:
-        del accelerators  # unused
-
-        if instance_type is None:
-            # fallback to manually specified region/zones
-            regions = cls.regions()
-        else:
-            regions = service_catalog.get_region_zones_for_instance_type(
-                instance_type, use_spot, clouds='azure')
+        regions = cls.regions_with_offering(instance_type,
+                                            accelerators,
+                                            use_spot,
+                                            region=None,
+                                            zone=None)
         for region in regions:
             yield region, region.zones
 
@@ -159,18 +198,28 @@ class Azure(clouds.Cloud):
         return service_catalog.get_accelerators_from_instance_type(
             instance_type, clouds='azure')
 
+    @classmethod
+    def get_vcpus_from_instance_type(
+        cls,
+        instance_type: str,
+    ) -> Optional[float]:
+        return service_catalog.get_vcpus_from_instance_type(instance_type,
+                                                            clouds='azure')
+
+    @classmethod
+    def get_zone_shell_cmd(cls) -> Optional[str]:
+        return None
+
     def make_deploy_resources_variables(
             self, resources: 'resources.Resources',
             region: Optional['clouds.Region'],
-            zones: Optional[List['clouds.Zone']]) -> Dict[str, str]:
+            zones: Optional[List['clouds.Zone']]) -> Dict[str, Optional[str]]:
         if region is None:
             assert zones is None, (
                 'Set either both or neither for: region, zones.')
             region = self._get_default_region()
 
         region_name = region.name
-        # Azure does not support specific zones.
-        zones = []
 
         r = resources
         assert not r.use_spot, \
@@ -190,7 +239,8 @@ class Azure(clouds.Cloud):
             'custom_resources': custom_resources,
             'use_spot': r.use_spot,
             'region': region_name,
-            'zones': zones,
+            # Azure does not support specific zones.
+            'zones': None,
             **image_config
         }
 
@@ -228,9 +278,13 @@ class Azure(clouds.Cloud):
         assert len(accelerators) == 1, resources
         acc, acc_count = list(accelerators.items())[0]
         (instance_list, fuzzy_candidate_list
-        ) = service_catalog.get_instance_type_for_accelerator(acc,
-                                                              acc_count,
-                                                              clouds='azure')
+        ) = service_catalog.get_instance_type_for_accelerator(
+            acc,
+            acc_count,
+            use_spot=resources.use_spot,
+            region=resources.region,
+            zone=resources.zone,
+            clouds='azure')
         if instance_list is None:
             return ([], fuzzy_candidate_list)
         return (_make(instance_list), fuzzy_candidate_list)
@@ -253,22 +307,20 @@ class Azure(clouds.Cloud):
                     f'{azure_token_cache_file} does not exist.' + help_str)
 
         try:
-            output = _run_output('az account show --output=json')
+            _run_output('az --version')
         except subprocess.CalledProcessError:
             return False, (
-                'Azure CLI returned error. Run the following commands '
-                'under sky folder:'
-                # TODO(zhwu): after we publish sky to PyPI, change this to
-                # `pip install sky[azure]`
-                '\n      $ pip install .[azure]'
+                # TODO(zhwu): Change the installation hint to from PyPI.
+                'Azure CLI returned error. Run the following commands:'
+                '\n      $ pip install skypilot[azure]'
                 '\n    Credentials may also need to be set.' + help_str)
-        # If Azure is properly logged in, this will return something like:
-        #   {"id": ..., "user": ...}
-        # and if not, it will return:
-        #   Please run 'az login' to setup account.
-        if output.startswith('{'):
-            return True, None
-        return False, 'Azure credentials not set.' + help_str
+        # If Azure is properly logged in, this will return the account email
+        # address + subscription ID.
+        try:
+            self.get_current_user_identity()
+        except exceptions.CloudUserIdentityError:
+            return False, 'Azure credential is not set.' + help_str
+        return True, None
 
     def get_credential_file_mounts(self) -> Dict[str, str]:
         """Returns a dict of credential file paths to mount paths."""
@@ -281,8 +333,49 @@ class Azure(clouds.Cloud):
         return service_catalog.instance_type_exists(instance_type,
                                                     clouds='azure')
 
-    def region_exists(self, region: str) -> bool:
-        return service_catalog.region_exists(region, 'azure')
+    def accelerator_in_region_or_zone(self,
+                                      accelerator: str,
+                                      acc_count: int,
+                                      region: Optional[str] = None,
+                                      zone: Optional[str] = None) -> bool:
+        return service_catalog.accelerator_in_region_or_zone(
+            accelerator, acc_count, region, zone, 'azure')
+
+    def get_current_user_identity(self) -> Optional[str]:
+        """Returns the cloud user identity."""
+        # This returns the user's email address + [subscription_id].
+        retry_cnt = 0
+        while True:
+            retry_cnt += 1
+            try:
+                import knack  # pylint: disable=import-outside-toplevel
+                account_email = azure.get_current_account_user()
+                break
+            except (FileNotFoundError, knack.util.CLIError) as e:
+                error = exceptions.CloudUserIdentityError(
+                    'Failed to get activated Azure account.\n'
+                    '  Reason: '
+                    f'{common_utils.format_exception(e, use_bracket=True)}')
+                if retry_cnt <= _MAX_IDENTITY_FETCH_RETRY:
+                    logger.debug(f'{error}.\nRetrying...')
+                    continue
+                with ux_utils.print_exception_no_traceback():
+                    raise error from None
+            except Exception as e:  # pylint: disable=broad-except
+                with ux_utils.print_exception_no_traceback():
+                    raise exceptions.CloudUserIdentityError(
+                        'Failed to get Azure user identity with unknown '
+                        f'exception.\n'
+                        '  Reason: '
+                        f'{common_utils.format_exception(e, use_bracket=True)}'
+                    ) from e
+        try:
+            project_id = self.get_project_id()
+        except (ModuleNotFoundError, RuntimeError) as e:
+            with ux_utils.print_exception_no_traceback():
+                raise exceptions.CloudUserIdentityError(
+                    'Failed to get Azure project ID.') from e
+        return f'{account_email} [subscription_id={project_id}]'
 
     @classmethod
     def get_project_id(cls, dryrun: bool = False) -> str:
