@@ -8,6 +8,7 @@ from sky import spot
 from sky.backends import backend_utils
 from sky.utils import accelerator_registry
 from sky.utils import schemas
+from sky.utils import tpu_utils
 from sky.utils import ux_utils
 
 logger = sky_logging.init_logger(__name__)
@@ -57,7 +58,7 @@ class Resources:
         disk_size: Optional[int] = None,
         region: Optional[str] = None,
         zone: Optional[str] = None,
-        image_id: Optional[str] = None,
+        image_id: Union[Dict[str, str], str, None] = None,
     ):
         self._version = self._VERSION
         self._cloud = cloud
@@ -74,11 +75,6 @@ class Resources:
             self._spot_recovery = spot_recovery.upper()
 
         if disk_size is not None:
-            if disk_size < 50:
-                with ux_utils.print_exception_no_traceback():
-                    raise ValueError(
-                        'OS disk size must be larger than 50GB. Got: '
-                        f'{disk_size}.')
             if round(disk_size) != disk_size:
                 with ux_utils.print_exception_no_traceback():
                     raise ValueError(
@@ -88,6 +84,10 @@ class Resources:
             self._disk_size = _DEFAULT_DISK_SIZE_GB
 
         self._image_id = image_id
+        if isinstance(image_id, str):
+            self._image_id = {self._region: image_id}
+        elif isinstance(image_id, dict) and None in image_id:
+            self._image_id = {self._region: image_id[None]}
 
         self._set_accelerators(accelerators, accelerator_args)
 
@@ -98,6 +98,29 @@ class Resources:
         self._try_validate_image_id()
 
     def __repr__(self) -> str:
+        """Returns a string representation for display.
+
+        Examples:
+
+            >>> sky.Resources(accelerators='V100')
+            <Cloud>({'V100': 1})
+
+            >>> sky.Resources(accelerators='V100', use_spot=True)
+            <Cloud>([Spot], {'V100': 1})
+
+            >>> sky.Resources(accelerators='V100',
+            ...     use_spot=True, instance_type='p3.2xlarge')
+            AWS(p3.2xlarge[Spot], {'V100': 1})
+
+            >>> sky.Resources(accelerators='V100', instance_type='p3.2xlarge')
+            AWS(p3.2xlarge, {'V100': 1})
+
+            >>> sky.Resources(instance_type='p3.2xlarge')
+            AWS(p3.2xlarge, {'V100': 1})
+
+            >>> sky.Resources(disk_size=100)
+            <Cloud>(disk_size=100)
+        """
         accelerators = ''
         accelerator_args = ''
         if self.accelerators is not None:
@@ -114,10 +137,33 @@ class Resources:
 
         image_id = ''
         if self.image_id is not None:
-            image_id = f', image_id={self.image_id!r}'
+            if None in self.image_id:
+                image_id = f', image_id={self.image_id[None]}'
+            else:
+                image_id = f', image_id={self.image_id!r}'
 
-        return (f'{self.cloud}({self._instance_type}{use_spot}'
-                f'{accelerators}{accelerator_args}{image_id})')
+        disk_size = ''
+        if self.disk_size != _DEFAULT_DISK_SIZE_GB:
+            disk_size = f', disk_size={self.disk_size}'
+
+        if self._instance_type is not None:
+            instance_type = f'{self._instance_type}'
+        else:
+            instance_type = ''
+
+        hardware_str = (
+            f'{instance_type}{use_spot}'
+            f'{accelerators}{accelerator_args}{image_id}{disk_size}')
+        # It may have leading ',' (for example, instance_type not set) or empty
+        # spaces.  Remove them.
+        while hardware_str and hardware_str[0] in (',', ' '):
+            hardware_str = hardware_str[1:]
+
+        cloud_str = '<Cloud>'
+        if self.cloud is not None:
+            cloud_str = f'{self.cloud}'
+
+        return f'{cloud_str}({hardware_str})'
 
     @property
     def cloud(self):
@@ -171,7 +217,7 @@ class Resources:
         return self._disk_size
 
     @property
-    def image_id(self) -> Optional[str]:
+    def image_id(self) -> Optional[Dict[str, str]]:
         return self._image_id
 
     def _set_accelerators(
@@ -204,7 +250,12 @@ class Resources:
                     except ValueError:
                         with ux_utils.print_exception_no_traceback():
                             raise ValueError(parse_error) from None
-            assert len(accelerators) == 1, accelerators
+
+            # Ignore check for the local cloud case.
+            # It is possible the accelerators dict can contain multiple
+            # types of accelerators for some on-prem clusters.
+            if not isinstance(self._cloud, clouds.Local):
+                assert len(accelerators) == 1, accelerators
 
             # Canonicalize the accelerator names.
             accelerators = {
@@ -222,7 +273,7 @@ class Resources:
                     accelerator_args = {}
                 use_tpu_vm = accelerator_args.get('tpu_vm', False)
                 if use_tpu_vm:
-                    backend_utils.check_gcp_cli_include_tpu_vm()
+                    tpu_utils.check_gcp_cli_include_tpu_vm()
                 if self.instance_type is not None and use_tpu_vm:
                     if self.instance_type != 'TPU-VM':
                         with ux_utils.print_exception_no_traceback():
@@ -244,6 +295,11 @@ class Resources:
     def is_launchable(self) -> bool:
         return self.cloud is not None and self._instance_type is not None
 
+    def need_cleanup_after_preemption(self) -> bool:
+        """Returns whether a spot resource needs cleanup after preeemption."""
+        assert self.is_launchable(), self
+        return self.cloud.need_cleanup_after_preemption(self)
+
     def _set_region_zone(self, region: Optional[str],
                          zone: Optional[str]) -> None:
         if region is None and zone is None:
@@ -254,13 +310,26 @@ class Resources:
                 raise ValueError(
                     'Cloud must be specified when region/zone are specified.')
 
-        # Validate whether region and zone exist in the catalog.
-        self._cloud.validate_region_zone(region, zone)
+        # Validate whether region and zone exist in the catalog, and set the
+        # region if zone is specified.
+        self._region, self._zone = self._cloud.validate_region_zone(
+            region, zone)
 
-        self._region = region
-        self._zone = zone
+    def get_offering_regions_for_launchable(self) -> List[clouds.Region]:
+        """Returns a set of `Region`s that can provision this Resources.
 
-    def _try_validate_instance_type(self):
+        Each `Region` has a list of `Zone`s that can provision this Resources.
+        """
+        assert self.is_launchable()
+        regions = self._cloud.regions_with_offering(self._instance_type,
+                                                    self.accelerators,
+                                                    self._use_spot,
+                                                    self._region, self._zone)
+        if self._image_id is not None and None not in self._image_id:
+            regions = [r for r in regions if r.name in self._image_id]
+        return regions
+
+    def _try_validate_instance_type(self) -> None:
         if self.instance_type is None:
             return
 
@@ -313,6 +382,7 @@ class Resources:
 
         if self.is_launchable() and not isinstance(self.cloud, clouds.GCP):
             # GCP attaches accelerators to VMs, so no need for this check.
+            acc_requested = self.accelerators
             acc_from_instance_type = (
                 self.cloud.get_accelerators_from_instance_type(
                     self._instance_type))
@@ -333,11 +403,14 @@ class Resources:
 
         # Validate whether accelerator is available in specified region/zone.
         acc, acc_count = list(acc_requested.items())[0]
+        # Fractional accelerators are temporarily bumped up to 1.
+        if 0 < acc_count < 1:
+            acc_count = 1
         if self.region is not None or self.zone is not None:
             if not self._cloud.accelerator_in_region_or_zone(
                     acc, acc_count, self.region, self.zone):
                 error_str = (f'Accelerator "{acc}" is not available in '
-                             '"{}" region/zone.')
+                             '"{}".')
                 if self.zone:
                     error_str = error_str.format(self.zone)
                 else:
@@ -384,28 +457,66 @@ class Resources:
             return
 
         if self.cloud is None:
-            raise ValueError('Cloud must be specified when image_id provided.')
+            with ux_utils.print_exception_no_traceback():
+                raise ValueError(
+                    'Cloud must be specified when image_id is provided.')
 
         if not self._cloud.is_same_cloud(
                 clouds.AWS()) and not self._cloud.is_same_cloud(clouds.GCP()):
-            raise ValueError(
-                'image_id is only supported for AWS and GCP, please '
-                'explicitly specify the cloud.')
+            with ux_utils.print_exception_no_traceback():
+                raise ValueError(
+                    'image_id is only supported for AWS and GCP, please '
+                    'explicitly specify the cloud.')
 
-        if self._cloud.is_same_cloud(clouds.AWS()) and self._region is None:
-            raise ValueError('image_id is only supported for AWS in a specific '
-                             'region, please explicitly specify the region.')
+        if self._region is not None:
+            if self._region not in self._image_id:
+                with ux_utils.print_exception_no_traceback():
+                    raise ValueError(
+                        f'image_id {self._image_id} should contain the image '
+                        f'for the specified region {self._region}.')
+            # Narrow down the image_id to the specified region.
+            self._image_id = {self._region: self._image_id[self._region]}
+
+        # Check the image_id's are valid.
+        for region, image_id in self._image_id.items():
+            if (image_id.startswith('skypilot:') and
+                    not self._cloud.is_image_tag_valid(image_id, region)):
+                region_str = f' ({region})' if region else ''
+                with ux_utils.print_exception_no_traceback():
+                    raise ValueError(
+                        f'Image tag {image_id} is not valid, please make sure'
+                        f' the tag exists in {self._cloud}{region_str}.')
+
+            if (self._cloud.is_same_cloud(clouds.AWS()) and
+                    not image_id.startswith('skypilot:') and region is None):
+                with ux_utils.print_exception_no_traceback():
+                    raise ValueError(
+                        'image_id is only supported for AWS in a specific '
+                        'region, please explicitly specify the region.')
+
+        # Validate the image exists and the size is smaller than the disk size.
+        for region, image_id in self._image_id.items():
+            # Check the image exists and get the image size.
+            # It will raise ValueError if the image does not exist.
+            image_size = self.cloud.get_image_size(image_id, region)
+            if image_size > self.disk_size:
+                with ux_utils.print_exception_no_traceback():
+                    raise ValueError(
+                        f'Image {image_id!r} is {image_size}GB, which is '
+                        f'larger than the specified disk_size: {self.disk_size}'
+                        ' GB. Please specify a larger disk_size to use this '
+                        'image.')
 
     def get_cost(self, seconds: float) -> float:
         """Returns cost in USD for the runtime in seconds."""
         hours = seconds / 3600
         # Instance.
         hourly_cost = self.cloud.instance_type_to_hourly_cost(
-            self._instance_type, self.use_spot)
+            self._instance_type, self.use_spot, self._region, self._zone)
         # Accelerators (if any).
         if self.accelerators is not None:
             hourly_cost += self.cloud.accelerators_to_hourly_cost(
-                self.accelerators, self.use_spot)
+                self.accelerators, self.use_spot, self._region, self._zone)
         return hourly_cost * hours
 
     def is_same_resources(self, other: 'Resources') -> bool:
@@ -490,8 +601,20 @@ class Resources:
             return False
         # self.zone <= other.zone
 
-        if (self.image_id is not None and self.image_id != other.image_id):
-            return False
+        if self.image_id is not None:
+            if other.image_id is None:
+                return False
+            if other.region is None:
+                # Current image_id should be a subset of other.image_id
+                if not self.image_id.items() <= other.image_id.items():
+                    return False
+            else:
+                this_image = (self.image_id.get(other.region) or
+                              self.image_id.get(None))
+                other_image = (other.image_id.get(other.region) or
+                               other.image_id.get(None))
+                if this_image != other_image:
+                    return False
 
         if (self._instance_type is not None and
                 self._instance_type != other.instance_type):
@@ -520,14 +643,27 @@ class Resources:
         # self <= other
         return True
 
-    def is_launchable_fuzzy_equal(self, other: 'Resources') -> bool:
-        """Whether the resources are the fuzzily same launchable resources."""
-        assert self.cloud is not None and other.cloud is not None
-        if not self.cloud.is_same_cloud(other.cloud):
-            return False
-        if self._instance_type is not None or other.instance_type is not None:
-            return self._instance_type == other.instance_type
-        return self.accelerators.keys() == other.accelerators.keys()
+    def should_be_blocked_by(self, blocked: 'Resources') -> bool:
+        """Whether this Resources matches the blocked Resources.
+
+        If a field in `blocked` is None, it should be considered as a wildcard
+        for that field.
+        """
+        is_matched = True
+        if (blocked.cloud is not None and
+                not self.cloud.is_same_cloud(blocked.cloud)):
+            is_matched = False
+        if (blocked.instance_type is not None and
+                self.instance_type != blocked.instance_type):
+            is_matched = False
+        if blocked.region is not None and self._region != blocked.region:
+            is_matched = False
+        if blocked.zone is not None and self._zone != blocked.zone:
+            is_matched = False
+        if (blocked.accelerators is not None and
+                self.accelerators != blocked.accelerators):
+            is_matched = False
+        return is_matched
 
     def is_empty(self) -> bool:
         """Is this Resources an empty request (all fields None)?"""
@@ -558,6 +694,17 @@ class Resources:
         assert len(override) == 0
         return resources
 
+    def valid_on_region_zones(self, region: str, zones: List[str]) -> bool:
+        """Returns whether this Resources is valid on given region and zones"""
+        if self.region is not None and self.region != region:
+            return False
+        if self.zone is not None and self.zone not in zones:
+            return False
+        if self.image_id is not None:
+            if None not in self.image_id and region not in self.image_id:
+                return False
+        return True
+
     @classmethod
     def from_yaml_config(cls, config: Optional[Dict[str, str]]) -> 'Resources':
         if config is None:
