@@ -23,16 +23,17 @@ update the ``_generate_node_name`` method and finally update the
 node provider.
 """
 
-from copy import deepcopy
-from typing import Any, Dict, List, Optional, Tuple, Union
-import logging
 import abc
-import time
+import logging
 import re
-from uuid import uuid4
+import time
+
 from collections import UserDict
+from copy import deepcopy
 from enum import Enum
 from functools import wraps
+from typing import Any, Dict, List, Optional, Tuple, Union
+from uuid import uuid4
 
 from googleapiclient.discovery import Resource
 from googleapiclient.errors import HttpError
@@ -48,6 +49,8 @@ MAX_POLLS = 12
 # considerably - this probably could be smaller
 # TPU deletion uses MAX_POLLS
 MAX_POLLS_TPU = MAX_POLLS * 8
+# Stopping instances can take several minutes, so we increase the timeout
+MAX_POLLS_STOP =  MAX_POLLS * 8
 POLL_INTERVAL = 5
 
 
@@ -134,6 +137,8 @@ class GCPNode(UserDict, metaclass=abc.ABCMeta):
 
     NON_TERMINATED_STATUSES = None
     RUNNING_STATUSES = None
+    STOPPED_STATUSES = None
+    STOPPING_STATUSES = None
     STATUS_FIELD = None
 
     def __init__(self, base_dict: dict, resource: "GCPResource", **kwargs) -> None:
@@ -147,6 +152,12 @@ class GCPNode(UserDict, metaclass=abc.ABCMeta):
     def is_terminated(self) -> bool:
         return self.get(self.STATUS_FIELD) not in self.NON_TERMINATED_STATUSES
 
+    def is_stopped(self) -> bool:
+        return self.get(self.STATUS_FIELD) in self.STOPPED_STATUSES
+
+    def is_stopping(self) -> bool:
+        return self.get(self.STATUS_FIELD) in self.STOPPING_STATUSES
+
     @property
     def id(self) -> str:
         return self["name"]
@@ -173,6 +184,8 @@ class GCPComputeNode(GCPNode):
     # https://cloud.google.com/compute/docs/instances/instance-life-cycle
     NON_TERMINATED_STATUSES = {"PROVISIONING", "STAGING", "RUNNING"}
     RUNNING_STATUSES = {"RUNNING"}
+    STOPPED_STATUSES = {"TERMINATED"}
+    STOPPING_STATUSES = {"STOPPING"}
     STATUS_FIELD = "status"
 
     def get_labels(self) -> dict:
@@ -196,13 +209,14 @@ class GCPTPUNode(GCPNode):
 
     NON_TERMINATED_STATUSES = {"CREATING", "STARTING", "RESTARTING", "READY"}
     RUNNING_STATUSES = {"READY"}
+    STOPPED_STATUSES = {"STOPPED"}
+    STOPPING_STATUSES = {"STOPPING"}
     STATUS_FIELD = "state"
 
     # SKY: get status of TPU VM for status filtering
     def get_status(self) -> str:
         return self.get(self.STATUS_FIELD)
 
-
     def get_labels(self) -> dict:
         return self.get("labels", {})
 
